@@ -1,5 +1,5 @@
 ;; tooltips.jl -- display frame-part keymap descriptions
-;; $Id: tooltips.jl,v 1.27 2000/11/27 18:17:18 jsh Exp $
+;; $Id: tooltips.jl,v 1.31 2002/04/21 03:39:34 jsh Exp $
 
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -29,6 +29,7 @@
 	  rep.system
 	  rep.regexp
 	  rep.io.timers
+	  sawfish.wm.commands
 	  sawfish.wm.custom
 	  sawfish.wm.misc
 	  sawfish.wm.windows
@@ -50,56 +51,28 @@
     "Display tooltips for window frames."
     :type boolean
     :group (misc tooltips)
-    :user-level novice
     :require sawfish.wm.ext.tooltips)
 
-  (defcustom tooltips-timeout-enabled nil
-    "Remove tooltips after a period of time."
-    :type boolean
-    :depends tooltips-enabled
-    :group (misc tooltips))
-
-  (defcustom tooltips-show-doc-strings t
+  (defcustom tooltips-show-doc-strings nil
     "Show full documentation in tooltips."
     :type boolean
     :depends tooltips-enabled
     :group (misc tooltips))
 
-  (defcustom tooltips-delay 500
-    "Number of milliseconds before displaying tooltips."
-    :type number
-    :range (0)
-    :depends tooltips-enabled
-    :group (misc tooltips))
+  (defvar tooltips-delay 500
+    "Number of milliseconds before displaying tooltips.")
 
-  (defcustom tooltips-timeout-delay 5000
-    "Number of milliseconds before removing tooltips."
-    :type number
-    :user-level expert
-    :range (0)
-    :depends tooltips-enabled
-    :group (misc tooltips))
+  (defvar tooltips-timeout-delay 5000
+    "Number of milliseconds before removing tooltips.")
 
-  (defcustom tooltips-font "-*-lucidatypewriter-medium-*-*-*-10-*-*-*-*-*-*-*"
-    "Font used to display tooltips."
-    :type font
-    :user-level expert
-    :depends tooltips-enabled
-    :group (misc tooltips))
+  (defvar tooltips-font nil
+    "Font used to display tooltips, or nil for default.")
 
-  (defcustom tooltips-background-color "grey85"
-    "Color used for the tooltips background"
-    :group (misc tooltips)
-    :user-level expert
-    :depends tooltips-enabled
-    :type color)
+  (defvar tooltips-background-color "grey85"
+    "Color used for the tooltips background")
 
-  (defcustom tooltips-foreground-color "black"
-    "Color used for the tooltips foreground"
-    :group (misc tooltips)
-    :user-level expert
-    :depends tooltips-enabled
-    :type color)
+  (defvar tooltips-foreground-color "black"
+    "Color used for the tooltips foreground")
 
 ;;; displaying tooltips
 
@@ -120,13 +93,12 @@
 			 (foreground . ,tooltips-foreground-color)
 			 (x-justify . left)
 			 (spacing . 2)
-			 (font . ,tooltips-font)))
+			 ,@(and tooltips-font tooltips-font)))
       (setq tooltips-displayed (or win t))
-      (when tooltips-timeout-enabled 
-	(setq tooltips-timer
-	      (make-timer remove-tooltip
-			  (quotient tooltips-timeout-delay 1000)
-			  (mod tooltips-timeout-delay 1000))))
+      (setq tooltips-timer
+	    (make-timer remove-tooltip
+			(quotient tooltips-timeout-delay 1000)
+			(mod tooltips-timeout-delay 1000)))
       (unless (in-hook-p 'pre-command-hook remove-tooltip)
 	(add-hook 'pre-command-hook remove-tooltip))))
 
@@ -197,9 +169,7 @@
     (let (doc)
       (if (and tooltips-show-doc-strings command
 	       (symbolp command)
-	       (progn
-		 (require 'rep.lang.doc)
-		 (setq doc (documentation command))))
+	       (setq doc (command-documentation command)))
 	  (_ doc)
 	(format nil "%S" command))))
 
