@@ -1,5 +1,5 @@
 ;; frames.jl -- handle window framing
-;; $Id: frames.jl,v 1.83 2001/03/01 04:35:44 jsh Exp $
+;; $Id: frames.jl,v 1.91 2002/10/20 08:07:50 jsh Exp $
 
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -23,8 +23,7 @@
 
     (compound-interface
      (structure-interface sawfish.wm.frames.subrs)
-     (export custom-set-frame-style
-	     after-setting-frame-option
+     (export after-setting-frame-option
 	     define-frame-type-mapper
 	     add-frame-style
 	     reload-frame-style
@@ -33,10 +32,14 @@
 	     rebuild-frames-with-style
 	     reframe-windows-with-style
 	     set-frame-style
+	     apply-frame-style
+	     apply-frame-style-and-save
 	     mark-frame-style-editable
 	     frame-style-editable-p
 	     window-type
 	     set-window-type
+	     push-window-type
+	     pop-window-type
 	     window-type-remove-title
 	     window-type-remove-border
 	     window-type-add-title
@@ -57,15 +60,14 @@
 	  rep.io.files
 	  rep.io.timers
 	  sawfish.wm.frames.subrs
-	  sawfish.wm.windows.subrs
+	  sawfish.wm.windows
 	  sawfish.wm.misc
 	  sawfish.wm.custom
 	  sawfish.wm.commands
 	  sawfish.wm.events
 	  sawfish.wm.gaol
 	  sawfish.wm.session.init
-	  sawfish.wm.workspace
-	  sawfish.wm.state.maximize)
+	  sawfish.wm.workspace)
 
   ;; Commentary:
 
@@ -85,6 +87,9 @@
   ;;	unframed		no frame at all
   ;;	icon
   ;;	dock
+  ;;	menu, toolbar
+  ;;	utility
+  ;;	splash
 
   ;; There is also a similar concept of frame types. The window type
   ;; never changes (unless the user explicitly does so). But the frame
@@ -133,35 +138,27 @@ that overrides settings set elsewhere.")
     "Default frame style:"
     :type frame-style
     :widget-flags (expand-vertically)
-    :user-level novice
     :group appearance
     :after-set (lambda () (after-setting-default-frame)))
 
   (defcustom reload-themes-when-changed t
     "Automatically reload themes when they are updated."
     :type boolean
-    :user-level expert
     :group misc)
 
-  (defcustom frame-type-fallback-alist
+  (defvar frame-type-fallback-alist
     '((transient . default)
       (shaped . default)
       (shaped-transient . shaped)
       (shaded . shaped)
       (shaded-transient . shaped-transient)
       (icon . shaped-transient)
-      (dock . icon))
-    "Frame type fallbacks:"
-    :tooltip "Associate frame types with type to try if the theme doesn't \
-implement the requested type."
-    :type (alist ((symbol default shaped transient
-			  shaped-transient icon doc) "From")
-		 ((symbol default shaped transient
-			  shaped-transient icon doc) "To"))
-    :widget-flags (expand-vertically framed)
-    :group appearance
-    :user-level expert
-    :after-set (lambda () (after-setting-frame-option)))
+      (dock . icon)
+      (utility . transient)
+      (toolbar . utility)
+      (menu . utility)
+      (splash . unframed))
+    "Frame type fallbacks.")
 
   (defvar theme-update-interval 60
     "Number of seconds between checking if theme files have been modified.")
@@ -215,15 +212,10 @@ deciding which frame type to ask a theme to generate.")
     :group appearance
     :type font
     :widget-flags (expand-horizontally)
-    :user-level novice
     :after-set (lambda () (after-setting-frame-option)))
 
-  (defcustom default-bevel-percent nil
-    "Bevel intensity: \\wpercent."
-    :group appearance
-    :type (number 0 100)
-    :user-level expert
-    :after-set (lambda () (after-setting-frame-option)))
+  (defvar default-bevel-percent nil
+    "Bevel intensity as a percentage.")
 
 
 ;;; managing frame types
@@ -338,6 +330,27 @@ deciding which frame type to ask a theme to generate.")
       (call-window-hook 'window-state-change-hook w (list '(frame-style)))
       (reframe-window w)))
 
+  ;; return true iff successful
+  (define (apply-frame-style style)
+    (let ((old-style default-frame-style))
+      (condition-case nil
+	  (progn
+	    (setq default-frame-style style)
+	    (after-setting-default-frame)
+	    t)
+	(error
+	 (setq default-frame-style old-style)
+	 (after-setting-frame-option)
+	 nil))))
+
+  (define (save-current-frame-style)
+    (require 'sawfish.wm.customize)
+    (customize-set 'default-frame-style default-frame-style))
+
+  (define (apply-frame-style-and-save style)
+    (when (apply-frame-style style)
+      (save-current-frame-style)))
+
 
 ;;; editable frame styles
 
@@ -370,11 +383,34 @@ deciding which frame type to ask a theme to generate.")
 	    'default))))
 
   (define (set-window-type w type)
-    (unless (eq (window-get w 'type) type)
+    (if (window-get w 'type/saved)
+	(window-put w 'type/saved type)
+      (unless (eq (window-get w 'type) type)
+	(window-put w 'type type)
+	(call-window-hook 'window-state-change-hook w (list '(type)))
+	(reframe-window w))))
+
+  ;; XXX do something with KEY. (It's a unique symbol used to mark
+  ;; XXX the different users of these functions)
+
+  (define (push-window-type w type key)
+    (window-put w 'type/key key)
+    (unless (eq (window-get w 'type type))
+      (unless (window-get w 'type/saved)
+	(window-put w 'type/saved (window-get w 'type)))
       (window-put w 'type type)
       (call-window-hook 'window-state-change-hook w (list '(type)))
       (reframe-window w)))
 
+  (define (pop-window-type w key)
+    (when (and (window-get w 'type/saved)
+	       (eq (window-get w 'type/key) key))
+      (window-put w 'type (window-get w 'type/saved))
+      (window-put w 'type/saved nil)
+      (window-put w 'type/key nil))
+    (call-window-hook 'window-state-change-hook w (list '(type)))
+    (reframe-window w))
+
   (define (window-type-remove-title type)
     (case type
       ((default) 'transient)
@@ -573,6 +609,7 @@ deciding which frame type to ask a theme to generate.")
       ok-to-bind))
 
   (define ((cursor-for-frame-part part) w)
+    (require 'sawfish.wm.state.maximize)
     (if (frame-part-movable-p w part)
 	(case part
 	  ((top-border) 'top_side)
@@ -618,5 +655,13 @@ deciding which frame type to ask a theme to generate.")
 
   (make-timer frames-on-idle theme-update-interval)
 
-  (sm-add-saved-properties 'type 'ignored 'frame-style)
+  (add-hook 'sm-window-save-functions
+	    (lambda (w)
+	      (cond ((window-get w 'type/saved)
+		     (list (cons 'type (window-get w 'type/saved))))
+		    ((window-get w 'type)
+		     (list (cons 'type (window-get w 'type)))))))
+
+  (sm-add-saved-properties 'ignored 'frame-style)
+  (sm-add-restored-properties 'type)
   (add-swapped-properties 'frame-active-color 'frame-inactive-color))
