@@ -3,7 +3,7 @@ exec rep "$0" "$@"
 !#
 
 ;; sawmill-menu -- subprocess to handle menus
-;; $Id: sawfish-menu.jl,v 1.6 2000/12/19 23:05:43 jsh Exp $
+;; $Id: sawfish-menu.jl,v 1.7 2003/01/12 20:35:21 jsh Exp $
 
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -23,7 +23,7 @@ exec rep "$0" "$@"
 ;; along with sawmill; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
-(require 'gui.gtk)
+(require 'gui.gtk-2.gtk)
 (require 'rep.data.tables)
 
 
@@ -39,19 +39,16 @@ exec rep "$0" "$@"
 (define (group-id-ref id) (table-ref (fluid group-table) id))
 
 (define (create-menu spec #!optional bar)
-  (let* ((menu (if bar (gtk-menu-bar-new) (gtk-menu-new)))
-	 (accels (gtk-menu-ensure-uline-accel-group menu)))
+  (let* ((menu (if bar (gtk-menu-bar-new) (gtk-menu-new))))
 
     ;; Set the label of the menu item, handling underlined accelerators
     (define (label-menu-item item label-text #!optional shortcut)
-      (let* ((label (gtk-label-new label-text))
-	     (hbox (gtk-hbox-new nil 16))
-	     (hkey (gtk-label-parse-uline label label-text)))
+      (let* ((label (gtk-label-new-with-mnemonic label-text))
+	     (hbox (gtk-hbox-new nil 16)))
 	(gtk-box-pack-start hbox label nil nil t 0)
 	(when shortcut
 	  (let ((accel (gtk-label-new shortcut)))
 	    (gtk-box-pack-end hbox accel nil nil 0)))
-	(gtk-widget-add-accelerator item "activate_item" accels hkey 0 0)
 	(gtk-widget-show-all hbox)
 	(gtk-container-add item hbox)))
 
@@ -85,20 +82,20 @@ exec rep "$0" "$@"
 			     (group-id-set group item))
 			    (check
 			     (setq item (gtk-check-menu-item-new))
-			     (gtk-check-menu-item-set-show-toggle item t))
+			     (gtk-check-menu-item-set-active item t))
 			    (t (setq item (gtk-menu-item-new))))
 		      (label-menu-item item label shortcut)
 		      (when check
-			(gtk-check-menu-item-set-state item (cdr check)))
+			(gtk-check-menu-item-set-active item (cdr check)))
 		      (when insensitive
 			(gtk-widget-set-sensitive item nil))))
 
-		  (gtk-signal-connect
+		  (g-signal-connect
 		   item "activate" (lambda ()
 				     (setq menu-selected (car cell))))))
 	      (when item
-		(gtk-widget-lock-accelerators item)
-		((if bar gtk-menu-bar-append gtk-menu-append) menu item)
+		;;(gtk-widget-lock-accelerators item)
+		(gtk-menu-shell-append menu item)
 		(gtk-widget-show item))))
 	  spec)
     menu))
@@ -106,7 +103,7 @@ exec rep "$0" "$@"
 (define (popup-menu spec #!optional timestamp position)
   (let ((menu (let-fluids ((group-table (make-group-table)))
 		(create-menu spec))))
-    (gtk-signal-connect menu "deactivate" gtk-main-quit)
+    (g-signal-connect menu "deactivate" gtk-main-quit)
     (setq menu-selected nil)
     (gtk-menu-popup-interp menu nil nil 0 (or timestamp 0) position)
     (gtk-main)
