@@ -1,5 +1,5 @@
 /* events.c -- Event handling
-   $Id: events.c,v 1.143 2001/11/13 03:50:51 jsh Exp $
+   $Id: events.c,v 1.154 2003/02/22 07:44:47 jsh Exp $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -56,6 +56,9 @@ static repv saved_current_context_map;
 /* We need a ButtonRelease on this fp. */
 struct frame_part *clicked_frame_part;
 
+/* Set to true when the pointer is actively grabbed */
+static bool pointer_is_grabbed;
+
 static XID event_handler_context;
 
 static Atom xa_sawmill_timestamp;
@@ -376,7 +379,6 @@ static void
 button_press (XEvent *ev)
 {
     struct frame_part *fp;
-    Lisp_Window *w = 0;
 
     /* This works for both press and release events. The desired outcome
        is that press and motion events get the context map active when
@@ -388,28 +390,36 @@ button_press (XEvent *ev)
     record_mouse_position (ev->xbutton.x_root, ev->xbutton.y_root,
 			   ev->type, ev->xany.window);
 
-    fp = find_frame_part_by_window (ev->xbutton.window);
-    if (fp != 0)
+    /* Only let through events we're expecting */
+    if (pointer_is_grabbed
+	|| ev->xbutton.window != root_window
+	|| ev->xbutton.subwindow == None)
     {
-	w = fp->win;
-
 	if (ev->type == ButtonPress)
-	    handle_fp_click (fp, ev);
-    }
+	{
+	    fp = find_frame_part_by_window (ev->xbutton.window);
+	    if (fp != 0)
+		handle_fp_click (fp, ev);
+	}
 
-    eval_input_event (current_context_map ());
+	eval_input_event (current_context_map ());
 
-    if (fp != 0 && !WINDOW_IS_GONE_P (w) && ev->type == ButtonRelease)
-    {
-	/* In case the event binding threw a non-local-exit, fake
-	   an unwind-protect thing */
-	repv old_throw = rep_throw_value;
-	rep_GC_root gc_old_throw;
-	rep_throw_value = rep_NULL;
-	rep_PUSHGC(gc_old_throw, old_throw);
-	handle_fp_click (fp, ev);
-	rep_POPGC;
-	rep_throw_value = old_throw;
+	if (ev->type == ButtonRelease)
+	{
+	    fp = find_frame_part_by_window (ev->xbutton.window);
+	    if (fp != 0 && !WINDOW_IS_GONE_P (fp->win))
+	    {
+		/* In case the event binding threw a non-local-exit, fake
+		   an unwind-protect thing */
+		repv old_throw = rep_throw_value;
+		rep_GC_root gc_old_throw;
+		rep_throw_value = rep_NULL;
+		rep_PUSHGC(gc_old_throw, old_throw);
+		handle_fp_click (fp, ev);
+		rep_POPGC;
+		rep_throw_value = old_throw;
+	    }
+	}
     }
 
     if (ev->type == ButtonRelease)
@@ -417,7 +427,7 @@ button_press (XEvent *ev)
 	button_press_mouse_x = button_press_mouse_y = -1;
 	button_press_window = 0;
 	/* The pointer is _always_ ungrabbed after a button-release */
-	XUngrabPointer (dpy, last_event_time);
+	ungrab_pointer ();
     }
 
     XAllowEvents (dpy, SyncPointer, last_event_time);
@@ -443,8 +453,14 @@ motion_notify (XEvent *ev)
 	record_mouse_position (x, y, ev->type, ev->xmotion.window);
     }
 
-    if (pointer_in_motion)
-	eval_input_event (current_context_map ());
+    /* Only let through events we're expecting */
+    if (pointer_is_grabbed
+	|| ev->xbutton.window != root_window
+	|| ev->xbutton.subwindow == None)
+    {
+	if (pointer_in_motion)
+	    eval_input_event (current_context_map ());
+    }
 
     XAllowEvents (dpy, SyncPointer, last_event_time);
 
@@ -462,7 +478,7 @@ property_notify (XEvent *ev)
     if (w != 0 && ev->xproperty.window == w->id)
     {
 	bool need_refresh = FALSE, changed = TRUE;
-	repv changed_states = Qnil;
+	repv changed_states = Qnil, prop;
 	rep_GC_root gc_w, gc_changed_states;
 
 	switch (ev->xproperty.atom)
@@ -553,11 +569,6 @@ property_notify (XEvent *ev)
 	    XGetWMNormalHints (dpy, w->id, &w->hints, &supplied);
 	    break;
 
-	case XA_WM_TRANSIENT_FOR:
-	    if (!XGetTransientForHint (dpy, w->id, &w->transient_for_hint))
-		w->transient_for_hint = 0;
-	    break;
-
 	default:
 	    if (ev->xproperty.atom == xa_wm_colormap_windows)
 	    {
@@ -580,6 +591,9 @@ property_notify (XEvent *ev)
 	rep_PUSHGC (gc_w, w_);
 	rep_PUSHGC (gc_changed_states, changed_states);
 
+	prop = x_atom_symbol (ev->xproperty.atom);
+	property_cache_invalidate (w_, prop);
+
 	if (need_refresh && w->reparented
 	    && w->property_change != 0 && !WINDOW_IS_GONE_P (w))
 	{
@@ -589,8 +603,7 @@ property_notify (XEvent *ev)
 	if (changed)
 	{
 	    Fcall_window_hook (Qproperty_notify_hook, rep_VAL(w),
-			       rep_list_2 (x_atom_symbol (ev->xproperty.atom),
-					   ev->xproperty.state
+			       rep_list_2 (prop, ev->xproperty.state
 					   == PropertyNewValue
 					   ? Qnew_value : Qdeleted), Qnil);
 	}
@@ -661,6 +674,7 @@ destroy_notify (XEvent *ev)
     if (w == 0 || ev->xdestroywindow.window != w->saved_id)
 	return;
     remove_window (w, Qt, Qnil);
+    property_cache_invalidate_window (rep_VAL (w));
     emit_pending_destroys ();
 }
 
@@ -781,7 +795,6 @@ unmap_notify (XEvent *ev)
 	}
 	Fcall_window_hook (Qunmap_notify_hook, rep_VAL(w), Qnil, Qnil);
 
-	focus_off_window (w);
 	XDeleteProperty (dpy, w->id, xa_wm_state);
 
 	/* Changed the window-handling model, don't let windows exist
@@ -890,6 +903,19 @@ leave_notify (XEvent *ev)
 
 static Lisp_Window *last_focused;
 
+static void
+report_focus_change (Lisp_Window *w)
+{
+    if (w != 0
+	&& w->focus_change != 0
+	&& !WINDOW_IS_GONE_P (w))
+    {
+	DB (("  calling focus change %p on %s\n",
+	     w->focus_change, rep_STR(w->name)));
+	w->focus_change (w);
+    }
+}
+
 static void
 focus_in (XEvent *ev)
 {
@@ -898,20 +924,16 @@ focus_in (XEvent *ev)
 	return;
     if (w != 0 && w->visible)
     {
+	Lisp_Window *old = focus_window;
 	focus_window = w;
-	if (pending_focus_window == w)
-	    pending_focus_window = 0;
 
 	if (last_focused != w)
 	{
 	    last_focused = w;
+	    if (old != 0)
+		report_focus_change (old);
 	    install_colormaps (w);
-	    if (w->focus_change != 0)
-	    {
-		DB (("  calling focus change %p on %s\n",
-		     w->focus_change, rep_STR(w->name)));
-		w->focus_change (w);
-	    }
+	    report_focus_change (w);
 	    if (!WINDOW_IS_GONE_P (w))
 	    {
 		Fcall_window_hook (Qfocus_in_hook, rep_VAL(w),
@@ -933,16 +955,14 @@ focus_out (XEvent *ev)
     if (w != 0 && ev->xfocus.detail != NotifyInferior)
     {
 	if (focus_window == w)
+	{
 	    focus_window = 0;
+	    report_focus_change (w);
+	}
+
 	if (last_focused == w)
 	{
 	    last_focused = 0;
-	    if (w->focus_change != 0)
-	    {
-		DB (("  calling focus change %p on %s\n",
-		     w->focus_change, rep_STR(w->name)));
-		w->focus_change (w);
-	    }
 	    if (!WINDOW_IS_GONE_P (w))
 	    {
 		Fcall_window_hook (Qfocus_out_hook, rep_VAL(w),
@@ -1031,13 +1051,14 @@ configure_request (XEvent *ev)
 		alist = Fcons (Fcons (Qstack, stack_data), alist);
 	    }
 	}
-	if ((mask & CWX) && (mask & CWY))
+	if ((mask & CWX) || (mask & CWY))
 	{
 	    int x = ev->xconfigurerequest.x;
 	    int y = ev->xconfigurerequest.y;
 	    alist = Fcons (Fcons (Qposition,
-				  Fcons (rep_MAKE_INT (x),
-					 rep_MAKE_INT (y))), alist);
+				  Fcons ((mask & CWX) ? rep_MAKE_INT (x) : Qnil,
+					 (mask & CWY) ? rep_MAKE_INT (y) : Qnil)),
+			   alist);
 	}
 	if ((mask & CWWidth) || (mask & CWHeight))
 	{
@@ -1071,6 +1092,11 @@ configure_request (XEvent *ev)
     }
 }
 
+static void
+gravity_notify (XEvent *ev)
+{
+}
+
 static void
 configure_notify (XEvent *ev)
 {
@@ -1194,6 +1220,19 @@ get_server_timestamp (void)
     return ev.xproperty.time;
 }
 
+void
+mark_pointer_grabbed (void)
+{
+    pointer_is_grabbed = TRUE;
+}
+
+void
+ungrab_pointer (void)
+{
+    pointer_is_grabbed = FALSE;
+    XUngrabPointer (dpy, last_event_time);
+}
+
 
 /* Window-local event handlers */
 
@@ -1282,9 +1321,14 @@ handle_input_mask(long mask)
 	if (xev.type == NoExpose || xev.type == GraphicsExpose)
 	    continue;
 
-	DB(("** Event: %s (win %lx)\n",
-	    xev.type < LASTEvent ? event_names[xev.type] : "unknown",
-	    (long)xev.xany.window));
+#ifdef DEBUG
+	do {
+	    Lisp_Window *w = x_find_window_by_id (xev.xany.window);
+	    DB(("** Event: %s (win %lx: %s)\n",
+		xev.type < LASTEvent ? event_names[xev.type] : "unknown",
+		(long)xev.xany.window, w ? (char *) rep_STR (w->name) : "unknown"));
+	} while (0);
+#endif
 
 	record_event_time (&xev);
 	current_x_event = &xev;
@@ -1460,9 +1504,9 @@ Return the window that received the current event, or the symbol
 }
 
 DEFUN("x-server-timestamp", Fx_server_timestamp, Sx_server_timestamp,
-      (repv from_server), rep_Subr1) /*
+      (repv from_server, repv store), rep_Subr2) /*
 ::doc:sawfish.wm.events#x-server-timestamp::
-x-server-timestamp [FROM-SERVER]
+x-server-timestamp [FROM-SERVER] [STORE]
 
 Return a recent X server timestamp, as an integer.
 
@@ -1473,6 +1517,8 @@ server, otherwise the most recent timestamp seen by the window manager
 {
     Time time = ((from_server == Qnil)
 		 ? last_event_time : get_server_timestamp ());
+    if (store != Qnil)
+	save_timestamp (time);
     return rep_make_long_uint (time);
 }
 
@@ -1514,6 +1560,7 @@ events_init (void)
     event_handlers[FocusIn] = focus_in;
     event_handlers[FocusOut] = focus_out;
     event_handlers[ConfigureRequest] = configure_request;
+    event_handlers[GravityNotify] = gravity_notify;
     event_handlers[ConfigureNotify] = configure_notify;
     event_handlers[ReparentNotify] = reparent_notify;
     event_handlers[CreateNotify] = create_notify;
@@ -1624,7 +1671,6 @@ events_init (void)
     rep_INTERN_SPECIAL(window_state_change_hook);
 
     rep_INTERN_SPECIAL(pointer_motion_threshold);
-    Fset (Qpointer_motion_threshold, rep_MAKE_INT (0));
 
     rep_INTERN(iconify_window);
     rep_INTERN(uniconify_window);
