@@ -1,5 +1,5 @@
 /* keys.c -- Key binding and evaluating (this should be called events.c)
-   $Id: keys.c,v 1.82 2001/02/27 07:15:04 jsh Exp $
+   $Id: keys.c,v 1.86 2002/10/20 04:05:40 jsh Exp $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -84,7 +84,7 @@ static u_long meta_mod, alt_mod, hyper_mod, super_mod;
 
 /* The user-customizable modifier; used for default key bindings. This
    shouldn't include any bits that don't have a fixed meaning. */
-static u_long wm_mod = ControlMask;
+static u_long wm_mod = EV_MOD_META;
 
 /* The X modifiers bound to the Num_Lock and Scroll_Lock keysyms */
 static u_long num_lock_mod, scroll_lock_mod;
@@ -210,6 +210,7 @@ translate_event(u_long *code, u_long *mods, XEvent *xev)
         }
 	else
 	    click_count = 1;
+
 	switch (click_count)
 	{
 	case 2:
@@ -228,7 +229,20 @@ translate_event(u_long *code, u_long *mods, XEvent *xev)
 	goto button;
 
     case ButtonRelease:
-	*code = EV_CODE_MOUSE_UP;
+	switch (click_count)
+	{
+	case 2:
+	    *code = EV_CODE_MOUSE_UP2;
+	    break;
+
+	case 3:
+	    *code = EV_CODE_MOUSE_UP3;
+	    break;
+
+	default:
+	    *code = EV_CODE_MOUSE_UP1;
+	}
+
     button:
 	*mods = xev->xbutton.state & ~all_lock_mask;
 	*mods |= EV_TYPE_MOUSE;
@@ -659,11 +673,14 @@ static struct key_def default_mods[] = {
 };
 
 static struct key_def default_codes[] = {
-    { "Click1",   EV_TYPE_MOUSE, EV_CODE_MOUSE_CLICK1 },
     { "Click",    EV_TYPE_MOUSE, EV_CODE_MOUSE_CLICK1 },
+    { "Click1",   EV_TYPE_MOUSE, EV_CODE_MOUSE_CLICK1 },
     { "Click2",   EV_TYPE_MOUSE, EV_CODE_MOUSE_CLICK2 },
     { "Click3",   EV_TYPE_MOUSE, EV_CODE_MOUSE_CLICK3 },
-    { "Off",      EV_TYPE_MOUSE, EV_CODE_MOUSE_UP },
+    { "Off",      EV_TYPE_MOUSE, EV_CODE_MOUSE_UP1 },
+    { "Off1",     EV_TYPE_MOUSE, EV_CODE_MOUSE_UP1 },
+    { "Off2",     EV_TYPE_MOUSE, EV_CODE_MOUSE_UP2 },
+    { "Off3",     EV_TYPE_MOUSE, EV_CODE_MOUSE_UP3 },
     { "Move",     EV_TYPE_MOUSE, EV_CODE_MOUSE_MOVE },
 
     { "SPC",      EV_TYPE_KEY, XK_space },
@@ -1086,7 +1103,7 @@ grab will be released first.
 	long e_mask = (rep_INTP(mask) ? rep_INT(mask)
 		       : get_event_mask (current_x_event->type));
 	if (current_x_event->type == ButtonPress)
-	    XUngrabPointer (dpy, last_event_time);
+	    ungrab_pointer ();
 	XSendEvent (dpy, w, prop == Qnil ? False : True,
 		    e_mask, current_x_event);
 	return Qt;
@@ -1441,6 +1458,17 @@ DEFSTRING(hyper_r, "Hyper_R");
 DEFSTRING(super_l, "Super_L");
 DEFSTRING(super_r, "Super_R");
 
+static void
+nconc (repv x, repv y)
+{
+    repv *ptr = &x;
+
+    while (rep_CONSP (*ptr))
+	ptr = rep_CDRLOC (*ptr);
+
+    *ptr = y;
+}
+
 static void
 find_meta(void)
 {
@@ -1539,17 +1567,17 @@ find_meta(void)
 
     if (meta_mod == alt_mod)
     {
-	meta_syms = Fnconc (rep_list_2 (meta_syms, alt_syms));
+	nconc (meta_syms, alt_syms);
 	alt_syms = meta_syms;
     }
     if (meta_mod == hyper_mod)
     {
-	meta_syms = Fnconc (rep_list_2 (meta_syms, hyper_syms));
+	nconc (meta_syms, hyper_syms);
 	hyper_syms = meta_syms;
     }
     if (meta_mod == super_mod)
     {
-	meta_syms = Fnconc (rep_list_2 (meta_syms, super_syms));
+	nconc (meta_syms, super_syms);
 	super_syms = meta_syms;
     }
 
